//BeOSRADIO Probe

#include "RadioProbeLooper.h"
#include "RadioProbeConstants.h"
#include <String.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <NetKit.h>
#include <NetworkKit.h>
#include <Debug.h>
#include <File.h>
#include <Path.h>
#include <FindDirectory.h>
#include <Entry.h>
#include <Roster.h>
#include <Handler.h>

RadioProbeLooper::~RadioProbeLooper()
{
	mList.DoForEach(RadioProbeLooper::Delete);
}

void RadioProbeLooper::MessageReceived(BMessage *msg)
{
	BString artist;
	BString title;
	status_t err;
	BHandler *view;
	const char *server;
	uint16 port;
	bool enable;

	switch(msg->what)
	{
		case P_LAUNCH_PLAYLIST:
		{
			LaunchPlayList();
		}
		break;

		case P_UPDATE_CURRENT_SONG:
		{
			if (msg->FindPointer("view", (void **)&view) < B_OK)
				return;
		
			if ((err = GetCurrentSong(title, artist)) < B_OK)
			{
				title.SetTo("(No connection)");
				artist.SetTo("(");
				artist.Append(strerror(err));
				artist.Append(")");
			}
			
			BMessage info(P_UPDATE_CURRENT_SONG);
			BMessenger messenger(view);
			if (info.AddString("title", title.String()) == B_OK &&
				info.AddString("artist", artist.String()) == B_OK)
				messenger.SendMessage(&info);
		}
		break;
	
		case P_SET_SERVER:
		{
			if ((msg->FindBool("enable", &enable) == B_OK) &&
				enable &&
				(msg->FindString("server", &server) == B_OK) &&
				(msg->FindInt16("port", (int16 *)&port) == B_OK))
			{
				mServer.SetTo(server);
				mPort = port;
			}
			else
			{
				mServer.SetTo("www.beosradio.com");
				mPort = 80;
			}
		}
		break;

		default:
			BLooper::MessageReceived(msg);
	}
}

status_t RadioProbeLooper::GetCurrentSong(BString& title, BString& artist)
{
	const char *request = "GET http://www.beosradio.com/what_is_playing/last_five HTTP/1.0\r\n"
							"User-Agent: BeOSRADIO Probe/1.00\r\n"
							"\r\n";
	const int32 BUFFER_SIZE = 4096;
	BNetEndpoint http;
	BString page;
	BString *song_info;
	char *buffer;
	status_t err;
	int32 begin, end, len;
	bool found = false;

	try
	{
		buffer = new char[BUFFER_SIZE];
	}
	catch(...)
	{
		return B_NO_MEMORY;
	}
	
	http.SetTimeout(10000000);
	if (((err = http.Connect(mServer.String(), mPort)) < B_OK) ||
		((err = http.Send(request, strlen(request))) < B_OK))
		goto ReadError;

	//Skip the HTTP header
	ReadMore:
	if ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		char *p = buffer;
		char *max = buffer+err-3;
	
		while(p < max)
		{
			if (*p == '\r' && *(p+1) == '\n')
			{
				p += 2;
				if (*p == '\r' && *(p+1) == '\n')
				{
					found = true;
					break;	
				}
			}
			else if (*p == '\n' && *(p+1) == '\n')
			{
				found = true;
				break;
			}
			++p;
		}
		
#if DEBUG
		*(p + 1) = '\0';
		printf("%s", buffer);
#endif
		
		if (found)
		{
			p+=2;
			
			if (err-(p-buffer) > 0)
			{
				try
				{
					page.Append(p, err-(p-buffer));
				}
				catch(...)
				{
					err = B_NO_MEMORY;
					goto ReadError;
				}
			}
		}
		else
			goto ReadMore;
	}
	else
	{
		if (err == B_OK)
			err = B_FILE_NOT_FOUND;
		goto ReadError;
	}

	//Receive the play list
	while((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		try
		{
			page.Append(buffer, err);
		}
		catch(...)
		{
			err = B_NO_MEMORY;
			goto ReadError;
		}
	}
	http.Close();
	
	if (err < 0)
		goto ReadError;
	
	len = page.Length();
	if (err == 0 && len == 0)
	{
		err = B_TIMED_OUT;
		goto ReadError;
	}

#if DEBUG
	printf("\n%s", page.String());
#endif
	
	//Parse the playlist for the songs and add them to mList
	mList.DoForEach(RadioProbeLooper::Delete);
	mList.MakeEmpty();
	begin = 0;
	err = B_OK;
		
	while(begin < len-1)
	{
		if ((end = page.FindFirst("\n", begin)) <= 0)
			break;

		song_info = NULL;	
		try
		{
			song_info = new BString(B_EMPTY_STRING);
			song_info->SetTo(page.String()+begin, end-begin);
			if (mList.AddItem((void *)song_info) == false)
				throw B_NO_MEMORY;
		}
		catch(...)
		{
			delete song_info;
			err = B_NO_MEMORY;
			goto ReadError;	
		}
		
		begin = end + 1;
	}
	
	//return the first song in the list
	song_info = (BString *)mList.ItemAt(0);
	if ((song_info == NULL) || (end = song_info->FindLast(";")) < 0)
	{
		artist.SetTo("(none)");
		title.SetTo("(none)");
	}
	else
	{
		const char *s = song_info->String();
		artist.SetTo(s, end);
		if (*s == ';')
			artist.RemoveFirst(";");
		title.SetTo(s+end+1, song_info->Length()-end-1);
	}

	printf("\nThe current song is <%s> by <%s>.\n", title.String(), artist.String());
	
	ReadError:
		http.Close();
		delete [] buffer;
		return err;
}

bool RadioProbeLooper::Delete(void *object)
{
	BString *s = (BString *)object;
	delete s;
	return false;
}

status_t RadioProbeLooper::LaunchPlayList()
{
	const char *request = "GET http://www.beosradio.com/playlist.m3u HTTP/1.0\r\n"
							"User-Agent: BeOSRADIO Probe/1.00\r\n"
							"\r\n";
	BFile file;
	BPath path;
	BNetEndpoint http;
	status_t err;
	char *buffer = NULL;
	bool found = false;
	const int32 BUFFER_SIZE = 4096;
	entry_ref ref;
	BEntry entry;

	if (((err = find_directory(B_COMMON_TEMP_DIRECTORY, &path)) < B_OK) ||
		((err = path.Append("playlist.m3u")) < B_OK))
		return err;
	
	try
	{
		buffer = new char[BUFFER_SIZE];
	}
	catch(...)
	{
		return B_NO_MEMORY;	
	}

	//BNetDebug::Enable(true);

	http.SetTimeout(10000000);
	
	if (((err = file.SetTo(path.Path(), B_WRITE_ONLY | B_ERASE_FILE | B_CREATE_FILE)) < B_OK) ||
		((err = http.Connect(mServer.String(), mPort)) < B_OK) ||
		((err = http.Send(request, strlen(request))) < B_OK))
		goto ReadError;

	ReadMore:

	if ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		char *p = buffer;
		char *max = buffer+err-3;
	
		while(p < max)
		{
			if (*p == '\r' && *(p+1) == '\n')
			{
				p += 2;
				if (*p == '\r' && *(p+1) == '\n')
				{
					found = true;
					break;	
				}
			}
			else if (*p == '\n' && *(p+1) == '\n')
			{
				found = true;
				break;
			}
			++p;
		}
		
#if DEBUG
		*(p + 1) = '\0';
		printf("%s", buffer);
#endif
		
		if (found)
		{
			p+=2;
			
			if (err-(p-buffer) > 0)
				if ((err = file.Write(p, err-(p-buffer))) < B_OK)
					goto ReadError;
		}
		else
			goto ReadMore;
	}
	else
	{
		if (err == B_OK)
			err = B_FILE_NOT_FOUND;
		goto ReadError;
	}

	while ((err = http.Receive(buffer, BUFFER_SIZE, 0)) > 0)
	{
		if ((err = file.Write(buffer, err)) < B_OK)
			break;
	}

	file.Unset();

	
		
	if ((err = be_roster->FindApp("text/x-playlist", &ref)) == B_OK)
	{
		const char *argv[3];
		argv[0] = ref.name;
		argv[1] = path.Path();
		argv[2] = NULL;
		be_roster->Launch("text/x-playlist", 2, (char **)argv);
	}
	
	ReadError:
		//printf("%s\n", strerror(err));

		http.Close();
		file.Unset();
		delete [] buffer;
		if (err < B_OK)
			unlink(path.Path());
		return err;
}
