//BeOSRADIO Probe

#include "RadioProbeView.h"
#include "RadioProbeLooper.h"
#include "RadioProbeLink.h"
#include "RadioProbeProxy.h"
#include "RadioProbeConstants.h"

#include <Bitmap.h>
#include <Messenger.h>
#include <MessageRunner.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Menu.h>
#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <Dragger.h>
#include <SupportDefs.h>
#include <time.h>
#include <stdio.h>
#include "icon.c"

RadioProbeView::RadioProbeView(BRect frame)
	: BView(frame, "BeOSRADIO Probe", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS)
{
	Init();
}

RadioProbeView::RadioProbeView(BMessage *archive) : BView(archive)
{
	archive->FindMessage("RPProxyInfo", &mProxyInfo);
	Init();
}

void RadioProbeView::Init()
{
	app_info ai;
	BDragger *dragger;
	BRect frame;
	BMessage set_server(P_SET_SERVER);

	SetFont(be_bold_font);
	SetFontSize(12);
	SetViewColor(255, 152, 51);
	SetLowColor(255, 152, 51);

	mBitmap = new BBitmap(BRect(0, 0, kImageWidth-1, kImageHeight-1), kImageColorSpace);
	mBitmap->SetBits((const void *)kImageBits, sizeof(kImageBits), 0, kImageColorSpace);
	
	mTitle.SetTo("(Connecting)");
	mArtist.SetTo("(net_server)");
	
	mMessage.what = P_UPDATE_CURRENT_SONG;
	mMessage.AddPointer("view", (void *)this);

	mLooper = new RadioProbeLooper();

	RadioProbeLink *link = new RadioProbeLink(BRect(7, 50, 275, 62), mLooper);
	AddChild(link);

	if (be_app->GetAppInfo(&ai) < B_OK)
		throw B_ERROR;
		
	if (strcmp(ai.signature, RadioProbeAppSig) != 0)
	{
		frame = Bounds();
		dragger = new BDragger(BRect(frame.right-7,frame.bottom-7,frame.right,frame.bottom), this,  B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW);
		AddChild(dragger);
	}

	mLooper->Run();

	mProxyInfo.what = P_SET_SERVER;
	mLooper->PostMessage(&mProxyInfo);

	mRunner = new BMessageRunner(BMessenger(mLooper), &mMessage, 30000000);
}

RadioProbeView::~RadioProbeView()
{
	delete mRunner;
	if (mLooper->Lock())
		mLooper->Quit();
	delete mBitmap;
}

void RadioProbeView::AttachedToWindow()
{
	mLooper->PostMessage(&mMessage);
}

void RadioProbeView::FrameResized(float width, float height)
{
	BRect bounds = Window()->Bounds();

	ResizeTo(bounds.right, bounds.bottom);
	Invalidate();
}

void RadioProbeView::Draw(BRect updateRect)
{
	BRect frame = Frame();
	float bottom = min_c(frame.bottom, Window()->Bounds().bottom);
	int32 vpos;
	if ((frame.right - frame.left) >= 320)
		vpos = (int32)(bottom - frame.top - kImageHeight) / 2;
	else
		vpos = 0;
	
	DrawBitmap(mBitmap, BPoint(Bounds().right-kImageWidth, vpos));

	DrawString("Title:", BPoint(7,20));
	DrawString(mTitle.String(), BPoint(50,20));
	DrawString("Artist:", BPoint(7,37));
	DrawString(mArtist.String(), BPoint(50,37));
}

void RadioProbeView::MouseDown(BPoint where)
{
	BPoint location;
	uint32 buttons;
	BWindow *window;
	BRect frame;
	app_info ai;
	bool replicant;

	if ((be_app->GetAppInfo(&ai) == B_OK) &&
		(strcmp(ai.signature, RadioProbeAppSig) == 0))
		replicant = false;
	else
		replicant = true;

	window = Window();
	GetMouse(&location, &buttons);

	if ((window != NULL) && (buttons & B_SECONDARY_MOUSE_BUTTON))
	{
		BPopUpMenu *menu = new BPopUpMenu("Properties", false, false, B_ITEMS_IN_COLUMN);
		BMenuItem *item;

		item = new BMenuItem("About…", new BMessage(B_ABOUT_REQUESTED));
		item->SetTarget(this);
		menu->AddItem(item);

		menu->AddSeparatorItem();

		if (!replicant)
		{
			item = new BMenuItem("Always on Top", new BMessage(P_ALWAYS_ON_TOP));
			item->SetTarget(this);
			menu->AddItem(item);
			item->SetMarked(window->Feel() == B_FLOATING_ALL_WINDOW_FEEL);
	
			item = new BMenuItem("Show on All Workspaces", new BMessage(P_ALL_WORKSPACES));
			item->SetTarget(this);
			menu->AddItem(item);
			item->SetMarked(window->Workspaces() == B_ALL_WORKSPACES);
	
			BMenu *submenu = new BMenu("Title and Borders", B_ITEMS_IN_COLUMN);
			item->SetTarget(this);
			menu->AddItem(submenu);
	
			window_look look = window->Look();
			item = new BMenuItem("Normal", new BMessage(P_NORMAL_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_TITLED_WINDOW_LOOK);
	
			item = new BMenuItem("Small", new BMessage(P_SMALL_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_FLOATING_WINDOW_LOOK);
	
			item = new BMenuItem("Border", new BMessage(P_THICK_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_MODAL_WINDOW_LOOK);
	
			item = new BMenuItem("Outline", new BMessage(P_OUTLINE_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_BORDERED_WINDOW_LOOK);
	
			item = new BMenuItem("None", new BMessage(P_NO_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_NO_BORDER_WINDOW_LOOK);
		}

		item = new BMenuItem("Proxy…", new BMessage(P_PROXY_SETUP));
		item->SetTarget(this);
		menu->AddItem(item);

		if (!replicant)
		{
			menu->AddSeparatorItem();
	
			item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED));
			item->SetTarget(this);
			menu->AddItem(item);
		}

		menu->Go(ConvertToScreen(where), true, true, ConvertToScreen(BRect(where.x-1, where.y-1, where.x+1, where.y+1)), false);
		delete menu;
	}
	else if ((!replicant) & (buttons & B_PRIMARY_MOUSE_BUTTON))
	{
		BMessage drag_message('ARCV');
		
		frame = Bounds();
		frame.OffsetBy(location.x - frame.right, location.y - frame.bottom);
		if (Archive(&drag_message, false) == B_OK)
			DragMessage(&drag_message, frame, NULL);
	}
	else
		BView::MouseDown(where);
}

void RadioProbeView::MessageReceived(BMessage *msg)
{
	BWindow *window = Window();
	const char *title, *artist;

	switch(msg->what)
	{
		case P_UPDATE_CURRENT_SONG:
		{
			if ((msg->FindString("title", &title) == B_OK) &&
				(msg->FindString("artist", &artist) == B_OK))
			{
				mTitle.SetTo(title);
				mArtist.SetTo(artist);
				Invalidate();
			}
		}
		break;

		case P_ALWAYS_ON_TOP:
		{
			if (window->Feel() == B_FLOATING_ALL_WINDOW_FEEL)
				window->SetFeel(B_NORMAL_WINDOW_FEEL);
			else
				window->SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
		}
		break;

		case P_ALL_WORKSPACES:
		{
			if (window->Workspaces() == B_ALL_WORKSPACES)
				window->SetWorkspaces(B_CURRENT_WORKSPACE);
			else
				window->SetWorkspaces(B_ALL_WORKSPACES);
		}
		break;

		case P_NORMAL_BORDER:
			window->SetLook(B_TITLED_WINDOW_LOOK); break;

		case P_SMALL_BORDER:
			window->SetLook(B_FLOATING_WINDOW_LOOK); break;

		case P_THICK_BORDER:
			window->SetLook(B_MODAL_WINDOW_LOOK); break;

		case P_OUTLINE_BORDER:
			window->SetLook(B_BORDERED_WINDOW_LOOK); break;
			
		case P_NO_BORDER:
			window->SetLook(B_NO_BORDER_WINDOW_LOOK); break;

		case B_QUIT_REQUESTED:
		{
			app_info ai;
			BMessage note;
		
			if (be_app->GetAppInfo(&ai) == B_OK)
			{
				if (strcmp(ai.signature, RadioProbeAppSig) == 0)
					be_app->PostMessage(B_QUIT_REQUESTED);
			}
		}
		break;

		case B_ABOUT_REQUESTED:
		{
			struct tm tm_start = {0,0,18,31,11,100,0,365,-1};
			struct tm tm_end = {0,0,6,1,0,101,1,0,-1};
			char start[10], end[6];
			
			tzset();
			time_t start_time = mktime(&tm_start) - timezone;
			time_t end_time = mktime(&tm_end) - timezone;
			localtime_r(&start_time, &tm_start);
			localtime_r(&end_time, &tm_end);
			strftime(start, sizeof(start), "%H:%M to ", &tm_start);
			strftime(end, sizeof(end), "%H:%M", &tm_end);

			BString string;
			string.Append("BeOSRadio Probe v1.00\n" "Copyright © 2000, Paul Yu\n\n"
							"The 'Radio Antenna' icons belong to BeOSRADIO.\n\n"
							"BeOSRADIO is heard 7 days a week, 12 hours a day, from ");
			string.Append(start);
			string.Append(end);
			string.Append(" in your local area.\n");
		
			(new BAlert("About", string.String(), "Freeware"))->Go();
		}
		break;

		case P_PROXY_SETUP:
		{
			RadioProbeProxy *proxy_window = new RadioProbeProxy(this, &mProxyInfo);
			proxy_window->Show();
		}
		break;

		case P_SET_SERVER:
		{
			try
			{
				mProxyInfo = *msg;
				mLooper->PostMessage(msg);
			}
			catch(...)
			{
				/*	*/
			}
		}
		break;
				
		default:
			BView::MessageReceived(msg);
	}
}

status_t RadioProbeView::Archive(BMessage *into, bool deep) const
{
	status_t err;
	if (((err = BView::Archive(into, false)) == B_OK) &&
		((err = into->AddMessage("RPProxyInfo", &mProxyInfo)) == B_OK))
		err = into->AddString("add_on", RadioProbeAppSig);
	return err;
}

BArchivable	*RadioProbeView::Instantiate(BMessage *from)
{
	if (validate_instantiation(from, "RadioProbeView"))
		return new RadioProbeView(from);
	return NULL;
}
